#include <QApplication>
#include <math.h>
#include "threedscatterselect.h"
#include <sstream>
#include <algorithm>
#include <assert.h>

using namespace std;


int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    app.setStyleSheet("* {font-family:arial;font-size:11px}");
    ThreeDScatterSelect demo;
    demo.show();
    return app.exec();
}


ThreeDScatterSelect::ThreeDScatterSelect(QWidget *parent) :
    QDialog(parent), random(3)
{
    xData = random.getSeries(20, 100, -10, 10);
    yData = random.getSeries(20, 100, -10, 10);
    zData = random.getSeries(20, 100, -10, 10);
    selected.resize(xData.len);

    setWindowTitle("Track Line with Data Labels");

    // Create the QChartViewer and draw the chart
    m_ChartViewer = new QChartViewer(this);
    drawChart(m_ChartViewer);

    // Set the window to be of the same size as the chart
    setFixedSize(m_ChartViewer->width(), m_ChartViewer->height());

    // Set up the mouseMovePlotArea handler for drawing the track cursor
    //connect(m_ChartViewer, SIGNAL(clicked(QMouseEvent*)), SLOT(onChartClicked(QMouseEvent*)));

    // Set up the mouseMovePlotArea handler for drawing the track cursor
    connect(m_ChartViewer, SIGNAL(mouseMoveChart(QMouseEvent*)),
        SLOT(onMouseMoveChart(QMouseEvent*)));
}

ThreeDScatterSelect::~ThreeDScatterSelect()
{
    delete m_ChartViewer->getChart();
}

//
// Draw the chart and display it in the given viewer
//
void ThreeDScatterSelect::drawChart(QChartViewer *viewer)
{
    // The x and y coordinates of the grid
    double dataX[] = {-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
    double dataY[] = {-10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10};

    // The values at the grid points. In this example, we will compute the values using the formula
    // z = x * sin(y) + y * sin(x).
    double dataZ[(int)(sizeof(dataX) / sizeof(dataX[0])) * (int)(sizeof(dataY) / sizeof(dataY[0]))];
    for(int yIndex = 0; yIndex < (int)(sizeof(dataY) / sizeof(dataY[0])); ++yIndex) {
        double y = dataY[yIndex];
        for(int xIndex = 0; xIndex < (int)(sizeof(dataX) / sizeof(dataX[0])); ++xIndex) {
            double x = dataX[xIndex];
            dataZ[yIndex * (int)(sizeof(dataX) / sizeof(dataX[0])) + xIndex] = x * sin(y) + y * sin(
                x);
        }
    }

    // Create a SurfaceChart object of size 720 x 600 pixels
    SurfaceChart *c = new SurfaceChart(720, 600);

    // Add a title to the chart using 20 points Times New Roman Italic font
    c->addTitle("Surface Energy Density   ", "timesi.ttf", 20);

    // Set the center of the plot region at (350, 280), and set width x depth x height to 360 x 360
    // x 270 pixels
    c->setPlotRegion(350, 280, 360, 360, 270);

    // Set the data to use to plot the chart
    c->setData(DoubleArray(dataX, (int)(sizeof(dataX) / sizeof(dataX[0]))), DoubleArray(dataY,
        (int)(sizeof(dataY) / sizeof(dataY[0]))), DoubleArray(dataZ, (int)(sizeof(dataZ) / sizeof(
        dataZ[0]))));

    // Spline interpolate data to a 80 x 80 grid for a smooth surface
    c->setInterpolation(80, 80);

    // Add a color axis (the legend) in which the left center is anchored at (645, 270). Set the
    // length to 200 pixels and the labels on the right side.
    c->setColorAxis(645, 270, Chart::Left, 200, Chart::Right);

    // Set the x, y and z axis titles using 10 points Arial Bold font
    c->xAxis()->setTitle("X (nm)", "arialbd.ttf", 10);
    c->yAxis()->setTitle("Y (nm)", "arialbd.ttf", 10);
    c->zAxis()->setTitle("Energy Density (J/m<*font,super*>2<*/font*>)", "arialbd.ttf", 10);

    // Output the chart
    viewer->setChart(c);
}

void ThreeDScatterSelect::trackLegend(SurfaceChart *c, int mouseX, int mouseY)
{
    // Clear the current dynamic layer and get the DrawArea object to draw on it.
    DrawArea *d = c->initDynamicLayer();

    DoubleArray xyzw = c->getValuesAtPixel(mouseX, mouseY);

    // Create the legend by joining the legend entries
    if (xyzw.len >= 3)
    {
        char buffer[8192];
        sprintf(buffer,
            "<*block,bgColor=FFFFCC,edgeColor=000000,margin=5*>x: %.2f<*br*>y: %.2f<*br*>z: %.2f<*/*>",
            xyzw[0], xyzw[1], xyzw[2]);

        TTFText *t = d->text(buffer, "arialbd.ttf", 8);
        t->draw(min(mouseX + 12, c->getWidth() - t->getWidth()), min(mouseY + 18, c->getHeight()
            - t->getHeight()), 0x000000, Chart::TopLeft);
        t->destroy();
    }
}

//
// User clicks on the QChartViewer
//
void ThreeDScatterSelect::onMouseMoveChart(QMouseEvent *event)
{
     trackLegend((SurfaceChart *)m_ChartViewer->getChart(), m_ChartViewer->getChartMouseX(), m_ChartViewer->getChartMouseY());
     m_ChartViewer->updateDisplay();
}
